# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"totensor": "000_utils.ipynb",
         "toarray": "000_utils.ipynb",
         "toL": "000_utils.ipynb",
         "to3dtensor": "000_utils.ipynb",
         "to2dtensor": "000_utils.ipynb",
         "to1dtensor": "000_utils.ipynb",
         "to3darray": "000_utils.ipynb",
         "to2darray": "000_utils.ipynb",
         "to1darray": "000_utils.ipynb",
         "to3d": "000_utils.ipynb",
         "to2d": "000_utils.ipynb",
         "to1d": "000_utils.ipynb",
         "to2dPlus": "000_utils.ipynb",
         "to3dPlus": "000_utils.ipynb",
         "to2dPlusTensor": "000_utils.ipynb",
         "to2dPlusArray": "000_utils.ipynb",
         "to3dPlusTensor": "000_utils.ipynb",
         "to3dPlusArray": "000_utils.ipynb",
         "todtype": "000_utils.ipynb",
         "bytes2size": "000_utils.ipynb",
         "bytes2GB": "000_utils.ipynb",
         "delete_all_in_dir": "000_utils.ipynb",
         "reverse_dict": "000_utils.ipynb",
         "is_tuple": "000_utils.ipynb",
         "itemify": "000_utils.ipynb",
         "isnone": "000_utils.ipynb",
         "exists": "000_utils.ipynb",
         "ifelse": "000_utils.ipynb",
         "is_not_close": "000_utils.ipynb",
         "test_not_close": "000_utils.ipynb",
         "test_type": "000_utils.ipynb",
         "test_ok": "000_utils.ipynb",
         "test_not_ok": "000_utils.ipynb",
         "test_error": "000_utils.ipynb",
         "assert_fn": "000_utils.ipynb",
         "test_gt": "000_utils.ipynb",
         "test_ge": "000_utils.ipynb",
         "test_lt": "000_utils.ipynb",
         "test_le": "000_utils.ipynb",
         "stack": "000_utils.ipynb",
         "stack_pad": "000_utils.ipynb",
         "random_shuffle": "000_utils.ipynb",
         "cat2int": "000_utils.ipynb",
         "cycle_dl": "000_utils.ipynb",
         "cycle_dl_to_device": "000_utils.ipynb",
         "cache_memmap": "000_utils.ipynb",
         "memmap2cache": "000_utils.ipynb",
         "get_func_defaults": "000_utils.ipynb",
         "get_idx_from_df_col_vals": "000_utils.ipynb",
         "get_sublist_idxs": "000_utils.ipynb",
         "flatten_list": "000_utils.ipynb",
         "display_pd_df": "000_utils.ipynb",
         "ttest": "000_utils.ipynb",
         "tscore": "000_utils.ipynb",
         "ttest_tensor": "000_utils.ipynb",
         "pcc": "000_utils.ipynb",
         "scc": "000_utils.ipynb",
         "a": "000_utils.ipynb",
         "b": "000_utils.ipynb",
         "remove_fn": "000_utils.ipynb",
         "npsave": "000_utils.ipynb",
         "np_save": "000_utils.ipynb",
         "permute_2D": "000_utils.ipynb",
         "random_normal": "000_utils.ipynb",
         "random_half_normal": "000_utils.ipynb",
         "random_normal_tensor": "000_utils.ipynb",
         "random_half_normal_tensor": "000_utils.ipynb",
         "clip_outliers": "000_utils.ipynb",
         "default_dpi": "000_utils.ipynb",
         "get_plot_fig": "000_utils.ipynb",
         "fig2buf": "000_utils.ipynb",
         "plot_scatter": "000_utils.ipynb",
         "jointplot_scatter": "000_utils.ipynb",
         "jointplot_kde": "000_utils.ipynb",
         "get_idxs": "000_utils.ipynb",
         "apply_cmap": "000_utils.ipynb",
         "torch_tile": "000_utils.ipynb",
         "to_tsfresh_df": "000_utils.ipynb",
         "pcorr": "000_utils.ipynb",
         "scorr": "000_utils.ipynb",
         "torch_diff": "000_utils.ipynb",
         "get_outliers_IQR": "000_utils.ipynb",
         "get_percentile": "000_utils.ipynb",
         "torch_clamp": "000_utils.ipynb",
         "torch_slice_by_dim": "000_utils.ipynb",
         "reduce_memory_usage": "000_utils.ipynb",
         "check_overlap": "000b_data.validation.ipynb",
         "leakage_finder": "000b_data.validation.ipynb",
         "balance_idx": "000b_data.validation.ipynb",
         "TrainValidTestSplitter": "000b_data.validation.ipynb",
         "get_splits": "000b_data.validation.ipynb",
         "TimeSplitter": "000b_data.validation.ipynb",
         "get_predefined_splits": "000b_data.validation.ipynb",
         "combine_split_data": "000b_data.validation.ipynb",
         "df2xy": "000c_data.preparation.ipynb",
         "SlidingWindow": "000c_data.preparation.ipynb",
         "SlidingWindowPanel": "000c_data.preparation.ipynb",
         "decompress_from_url": "001_data.external.ipynb",
         "get_UCR_univariate_list": "001_data.external.ipynb",
         "UCR_univariate_list": "001_data.external.ipynb",
         "get_UCR_multivariate_list": "001_data.external.ipynb",
         "UCR_multivariate_list": "001_data.external.ipynb",
         "UCR_list": "001_data.external.ipynb",
         "get_UCR_data": "001_data.external.ipynb",
         "NumpyTensor": "002_data.core.ipynb",
         "ToNumpyTensor": "002_data.core.ipynb",
         "TSTensor": "002_data.core.ipynb",
         "ToTSTensor": "002_data.core.ipynb",
         "ToFloat": "002_data.core.ipynb",
         "ToInt": "002_data.core.ipynb",
         "NumpyTensorBlock": "002_data.core.ipynb",
         "TSTensorBlock": "002_data.core.ipynb",
         "TorchDataset": "002_data.core.ipynb",
         "NumpyDataset": "002_data.core.ipynb",
         "TSDataset": "002_data.core.ipynb",
         "NumpyDatasets": "002_data.core.ipynb",
         "TSDatasets": "002_data.core.ipynb",
         "add_ds": "002_data.core.ipynb",
         "NumpyDatasets.add_dataset": "002_data.core.ipynb",
         "NumpyDatasets.add_test": "002_data.core.ipynb",
         "NumpyDatasets.add_unlabeled": "002_data.core.ipynb",
         "NumpyDataLoader": "002_data.core.ipynb",
         "show_tuple": "002_data.core.ipynb",
         "TSDataLoader": "002_data.core.ipynb",
         "NumpyDataLoaders": "002_data.core.ipynb",
         "TSDataLoaders": "002_data.core.ipynb",
         "get_ts_dls": "002_data.core.ipynb",
         "ToNumpyCategory": "003_data.preprocessing.ipynb",
         "OneHot": "003_data.preprocessing.ipynb",
         "TSStandardize": "003_data.preprocessing.ipynb",
         "torch.Tensor.mul_min": "003_data.preprocessing.ipynb",
         "TSTensor.mul_min": "003_data.preprocessing.ipynb",
         "NumpyTensor.mul_min": "003_data.preprocessing.ipynb",
         "torch.Tensor.mul_max": "003_data.preprocessing.ipynb",
         "TSTensor.mul_max": "003_data.preprocessing.ipynb",
         "NumpyTensor.mul_max": "003_data.preprocessing.ipynb",
         "TSNormalize": "003_data.preprocessing.ipynb",
         "TSClipOutliers": "003_data.preprocessing.ipynb",
         "TSRobustScaler": "003_data.preprocessing.ipynb",
         "TSDiff": "003_data.preprocessing.ipynb",
         "TSLog": "003_data.preprocessing.ipynb",
         "TSLogReturn": "003_data.preprocessing.ipynb",
         "TSIdentity": "003b_data.transforms.ipynb",
         "TSShuffle_HLs": "003b_data.transforms.ipynb",
         "TSShuffleSteps": "003b_data.transforms.ipynb",
         "TSMagAddNoise": "003b_data.transforms.ipynb",
         "TSMagMulNoise": "003b_data.transforms.ipynb",
         "random_curve_generator": "003b_data.transforms.ipynb",
         "random_cum_curve_generator": "003b_data.transforms.ipynb",
         "random_cum_noise_generator": "003b_data.transforms.ipynb",
         "random_cum_linear_generator": "003b_data.transforms.ipynb",
         "TSTimeNoise": "003b_data.transforms.ipynb",
         "TSMagWarp": "003b_data.transforms.ipynb",
         "TSTimeWarp": "003b_data.transforms.ipynb",
         "TSWindowWarp": "003b_data.transforms.ipynb",
         "TSMagScale": "003b_data.transforms.ipynb",
         "TSMagScalePerVar": "003b_data.transforms.ipynb",
         "TSRandomResizedCrop": "003b_data.transforms.ipynb",
         "TSRandomZoomIn": "003b_data.transforms.ipynb",
         "TSWindowSlicing": "003b_data.transforms.ipynb",
         "TSRandomZoomOut": "003b_data.transforms.ipynb",
         "TSRandomTimeScale": "003b_data.transforms.ipynb",
         "TSRandomTimeStep": "003b_data.transforms.ipynb",
         "TSBlur": "003b_data.transforms.ipynb",
         "TSSmooth": "003b_data.transforms.ipynb",
         "maddest": "003b_data.transforms.ipynb",
         "TSFreqDenoise": "003b_data.transforms.ipynb",
         "TSRandomFreqNoise": "003b_data.transforms.ipynb",
         "TSRandomResizedLookBack": "003b_data.transforms.ipynb",
         "TSVarOut": "003b_data.transforms.ipynb",
         "TSCutOut": "003b_data.transforms.ipynb",
         "TSTimeStepOut": "003b_data.transforms.ipynb",
         "TSRandomCropPad": "003b_data.transforms.ipynb",
         "TSMaskOut": "003b_data.transforms.ipynb",
         "TSTranslateX": "003b_data.transforms.ipynb",
         "TSRandomShift": "003b_data.transforms.ipynb",
         "TSHorizontalFlip": "003b_data.transforms.ipynb",
         "TSRandomTrend": "003b_data.transforms.ipynb",
         "TSRandomRotate": "003b_data.transforms.ipynb",
         "TSVerticalFlip": "003b_data.transforms.ipynb",
         "TSResize": "003b_data.transforms.ipynb",
         "TSRandomSize": "003b_data.transforms.ipynb",
         "TSRandomLowRes": "003b_data.transforms.ipynb",
         "TSDownUpScale": "003b_data.transforms.ipynb",
         "TSRandomDownUpScale": "003b_data.transforms.ipynb",
         "all_TS_randaugs": "003b_data.transforms.ipynb",
         "RandAugment": "003b_data.transforms.ipynb",
         "TestTfm": "003b_data.transforms.ipynb",
         "get_tfm_name": "003b_data.transforms.ipynb",
         "reduce_loss": "003c_data.mixed_augmentation.ipynb",
         "MixHandler1D": "003c_data.mixed_augmentation.ipynb",
         "MixUp1D": "003c_data.mixed_augmentation.ipynb",
         "CutMix1D": "003c_data.mixed_augmentation.ipynb",
         "TSImage": "003d_data.image.ipynb",
         "ToTSImage": "003d_data.image.ipynb",
         "TSToPlot": "003d_data.image.ipynb",
         "TSToMat": "003d_data.image.ipynb",
         "TSToGADF": "003d_data.image.ipynb",
         "TSToGASF": "003d_data.image.ipynb",
         "TSToMTF": "003d_data.image.ipynb",
         "TSToRP": "003d_data.image.ipynb",
         "TSToJRP": "003d_data.image.ipynb",
         "get_tabular_dls": "005_data.tabular.ipynb",
         "MixedDataLoaders": "006_data.mixed.ipynb",
         "get_mixed_dls": "006_data.mixed.ipynb",
         "MatthewsCorrCoefBinary": "007_metrics.ipynb",
         "get_task_metrics": "007_metrics.ipynb",
         "Learner.show_batch": "008_learner.ipynb",
         "Learner.save_all": "008_learner.ipynb",
         "load_all": "008_learner.ipynb",
         "load_learner_all": "008_learner.ipynb",
         "Recorder.plot_metrics": "008_learner.ipynb",
         "Learner.plot_metrics": "008_learner.ipynb",
         "Learner.show_probas": "008_learner.ipynb",
         "ts_learner": "008_learner.ipynb",
         "LookAhead": "009_optimizer.ipynb",
         "Ralamb": "009_optimizer.ipynb",
         "ralamb": "009_optimizer.ipynb",
         "RangerLars": "009_optimizer.ipynb",
         "rangerlars": "009_optimizer.ipynb",
         "Madam": "009_optimizer.ipynb",
         "Ranger": "009_optimizer.ipynb",
         "rangergc": "009_optimizer.ipynb",
         "AdaBelief": "009_optimizer.ipynb",
         "version_higher": "009_optimizer.ipynb",
         "adabelief": "009_optimizer.ipynb",
         "GamblersCallback": "010_callback.core.ipynb",
         "TransformScheduler": "010_callback.core.ipynb",
         "ShowGraphCallback2": "010_callback.core.ipynb",
         "ShowGraph": "010_callback.core.ipynb",
         "UBDAug": "010_callback.core.ipynb",
         "NoisyStudent": "011_callback.semi_supervised.ipynb",
         "get_layers": "100_models.utils.ipynb",
         "is_layer": "100_models.utils.ipynb",
         "is_linear": "100_models.utils.ipynb",
         "is_bn": "100_models.utils.ipynb",
         "is_conv_linear": "100_models.utils.ipynb",
         "is_affine_layer": "100_models.utils.ipynb",
         "is_conv": "100_models.utils.ipynb",
         "has_bias": "100_models.utils.ipynb",
         "has_weight": "100_models.utils.ipynb",
         "has_weight_or_bias": "100_models.utils.ipynb",
         "check_bias": "100_models.utils.ipynb",
         "check_weight": "100_models.utils.ipynb",
         "build_ts_model": "100_models.utils.ipynb",
         "build_tabular_model": "100_models.utils.ipynb",
         "count_parameters": "100_models.utils.ipynb",
         "build_model": "100_models.utils.ipynb",
         "create_model": "100_models.utils.ipynb",
         "create_tabular_model": "100_models.utils.ipynb",
         "get_clones": "100_models.utils.ipynb",
         "get_nf": "100_models.utils.ipynb",
         "split_model": "100_models.utils.ipynb",
         "seq_len_calculator": "100_models.utils.ipynb",
         "noop": "100b_models.layers.ipynb",
         "lin_zero_init": "100b_models.layers.ipynb",
         "SwishBeta": "100b_models.layers.ipynb",
         "same_padding1d": "100b_models.layers.ipynb",
         "Pad1d": "100b_models.layers.ipynb",
         "Conv1dSame": "100b_models.layers.ipynb",
         "same_padding2d": "100b_models.layers.ipynb",
         "Pad2d": "100b_models.layers.ipynb",
         "Conv2dSame": "100b_models.layers.ipynb",
         "Conv2d": "100b_models.layers.ipynb",
         "Chomp1d": "100b_models.layers.ipynb",
         "Conv1dCausal": "100b_models.layers.ipynb",
         "Conv1d": "100b_models.layers.ipynb",
         "SeparableConv1d": "100b_models.layers.ipynb",
         "AddCoords1d": "100b_models.layers.ipynb",
         "ConvBlock": "100b_models.layers.ipynb",
         "Conv": "100b_models.layers.ipynb",
         "ConvBN": "100b_models.layers.ipynb",
         "ConvIN": "100b_models.layers.ipynb",
         "CoordConv": "100b_models.layers.ipynb",
         "CoordConvBN": "100b_models.layers.ipynb",
         "SepConv": "100b_models.layers.ipynb",
         "SepConvBN": "100b_models.layers.ipynb",
         "SepConvIN": "100b_models.layers.ipynb",
         "SepCoordConv": "100b_models.layers.ipynb",
         "SepCoordConvBN": "100b_models.layers.ipynb",
         "ResBlock1dPlus": "100b_models.layers.ipynb",
         "SEModule1d": "100b_models.layers.ipynb",
         "Norm": "100b_models.layers.ipynb",
         "BN1d": "100b_models.layers.ipynb",
         "IN1d": "100b_models.layers.ipynb",
         "LambdaPlus": "100b_models.layers.ipynb",
         "Squeeze": "100b_models.layers.ipynb",
         "Unsqueeze": "100b_models.layers.ipynb",
         "Add": "100b_models.layers.ipynb",
         "Concat": "100b_models.layers.ipynb",
         "Permute": "100b_models.layers.ipynb",
         "Transpose": "100b_models.layers.ipynb",
         "View": "100b_models.layers.ipynb",
         "Reshape": "100b_models.layers.ipynb",
         "Max": "100b_models.layers.ipynb",
         "Noop": "100b_models.layers.ipynb",
         "Sharpen": "100b_models.layers.ipynb",
         "MaxPPVPool1d": "100b_models.layers.ipynb",
         "MPPV1d": "100b_models.layers.ipynb",
         "Temp_Scale": "100b_models.layers.ipynb",
         "Vector_Scale": "100b_models.layers.ipynb",
         "Matrix_Scale": "100b_models.layers.ipynb",
         "get_calibrator": "100b_models.layers.ipynb",
         "GAP1d": "100b_models.layers.ipynb",
         "GACP1d": "100b_models.layers.ipynb",
         "SqueezeExciteBlock": "100b_models.layers.ipynb",
         "create_pool_head": "100b_models.layers.ipynb",
         "pool_head": "100b_models.layers.ipynb",
         "create_pool_plus_head": "100b_models.layers.ipynb",
         "pool_plus_head": "100b_models.layers.ipynb",
         "create_mlp_head": "100b_models.layers.ipynb",
         "mlp_head": "100b_models.layers.ipynb",
         "create_conv_head": "100b_models.layers.ipynb",
         "conv_head": "100b_models.layers.ipynb",
         "heads": "100b_models.layers.ipynb",
         "change_model_head": "100b_models.layers.ipynb",
         "GaussianNoise": "100b_models.layers.ipynb",
         "gambler_loss": "100b_models.layers.ipynb",
         "CrossEntropyLossOneHot": "100b_models.layers.ipynb",
         "ttest_bin_loss": "100b_models.layers.ipynb",
         "ttest_reg_loss": "100b_models.layers.ipynb",
         "CenterLoss": "100b_models.layers.ipynb",
         "CenterPlusLoss": "100b_models.layers.ipynb",
         "FocalLoss": "100b_models.layers.ipynb",
         "get_acts_and_grads": "100c_models.explainability.ipynb",
         "get_attibution_map": "100c_models.explainability.ipynb",
         "ResBlock": "101_models.ResNet.ipynb",
         "ResNet": "101_models.ResNet.ipynb",
         "ResBlockPlus": "101b_models.ResNetPlus.ipynb",
         "ResNetPlus": "101b_models.ResNetPlus.ipynb",
         "InceptionModule": "102_models.InceptionTime.ipynb",
         "InceptionBlock": "102_models.InceptionTime.ipynb",
         "InceptionTime": "102_models.InceptionTime.ipynb",
         "InceptionModulePlus": "102b_models.InceptionTimePlus.ipynb",
         "InceptionBlockPlus": "102b_models.InceptionTimePlus.ipynb",
         "InceptionTimePlus": "102b_models.InceptionTimePlus.ipynb",
         "InCoordTime": "102b_models.InceptionTimePlus.ipynb",
         "XCoordTime": "102b_models.InceptionTimePlus.ipynb",
         "InceptionTimePlus17x17": "102b_models.InceptionTimePlus.ipynb",
         "InceptionTimePlus32x32": "102b_models.InceptionTimePlus.ipynb",
         "InceptionTimePlus47x47": "102b_models.InceptionTimePlus.ipynb",
         "InceptionTimePlus62x62": "102b_models.InceptionTimePlus.ipynb",
         "MultiInceptionTimePlus": "102b_models.InceptionTimePlus.ipynb",
         "MLP": "103_models.MLP.ipynb",
         "FCN": "103b_models.FCN.ipynb",
         "FCNPlus": "103c_models.FCNPlus.ipynb",
         "ResCNNBlock": "104_models.ResCNN.ipynb",
         "ResCNN": "104_models.ResCNN.ipynb",
         "RNN": "105_models.RNN.ipynb",
         "LSTM": "105_models.RNN.ipynb",
         "GRU": "105_models.RNN.ipynb",
         "RNNPlus": "105_models.RNNPlus.ipynb",
         "LSTMPlus": "105_models.RNNPlus.ipynb",
         "GRUPlus": "105_models.RNNPlus.ipynb",
         "XceptionModule": "106_models.XceptionTime.ipynb",
         "XceptionBlock": "106_models.XceptionTime.ipynb",
         "XceptionTime": "106_models.XceptionTime.ipynb",
         "XceptionModulePlus": "106b_models.XceptionTimePlus.ipynb",
         "XceptionBlockPlus": "106b_models.XceptionTimePlus.ipynb",
         "XceptionTimePlus": "106b_models.XceptionTimePlus.ipynb",
         "RNN_FCN": "107_models.RNN_FCN.ipynb",
         "LSTM_FCN": "107_models.RNN_FCN.ipynb",
         "GRU_FCN": "107_models.RNN_FCN.ipynb",
         "MRNN_FCN": "107_models.RNN_FCN.ipynb",
         "MLSTM_FCN": "107_models.RNN_FCN.ipynb",
         "MGRU_FCN": "107_models.RNN_FCN.ipynb",
         "RNN_FCNPlus": "107b_models.RNN_FCNPlus.ipynb",
         "LSTM_FCNPlus": "107b_models.RNN_FCNPlus.ipynb",
         "GRU_FCNPlus": "107b_models.RNN_FCNPlus.ipynb",
         "MRNN_FCNPlus": "107b_models.RNN_FCNPlus.ipynb",
         "MLSTM_FCNPlus": "107b_models.RNN_FCNPlus.ipynb",
         "MGRU_FCNPlus": "107b_models.RNN_FCNPlus.ipynb",
         "TransformerModel": "108_models.TransformerModel.ipynb",
         "ScaledDotProductAttention": "108c_models.TSTPlus.ipynb",
         "MultiHeadAttention": "108c_models.TSTPlus.ipynb",
         "TSTEncoderLayer": "108c_models.TSTPlus.ipynb",
         "TSTEncoder": "108c_models.TSTPlus.ipynb",
         "TST": "108b_models.TST.ipynb",
         "SinCosPosEncoding": "108c_models.TSTPlus.ipynb",
         "Coord2dPosEncoding": "108c_models.TSTPlus.ipynb",
         "Coord1dPosEncoding": "108c_models.TSTPlus.ipynb",
         "TSTPlus": "108c_models.TSTPlus.ipynb",
         "MultiTST": "108c_models.TSTPlus.ipynb",
         "SampaddingConv1D_BN": "109_models.OmniScaleCNN.ipynb",
         "build_layer_with_layer_parameter": "109_models.OmniScaleCNN.ipynb",
         "OmniScaleCNN": "109_models.OmniScaleCNN.ipynb",
         "get_Prime_number_in_a_range": "109_models.OmniScaleCNN.ipynb",
         "get_out_channel_number": "109_models.OmniScaleCNN.ipynb",
         "generate_layer_parameter_list": "109_models.OmniScaleCNN.ipynb",
         "WaveBlock": "110_models.mWDN.ipynb",
         "mWDN": "110_models.mWDN.ipynb",
         "generate_kernels": "111_models.ROCKET.ipynb",
         "apply_kernel": "111_models.ROCKET.ipynb",
         "apply_kernels": "111_models.ROCKET.ipynb",
         "ROCKET": "111_models.ROCKET.ipynb",
         "create_rocket_features": "111_models.ROCKET.ipynb",
         "xresnet1d18": "112_models.XResNet1d.ipynb",
         "xresnet1d34": "112_models.XResNet1d.ipynb",
         "xresnet1d50": "112_models.XResNet1d.ipynb",
         "xresnet1d101": "112_models.XResNet1d.ipynb",
         "xresnet1d152": "112_models.XResNet1d.ipynb",
         "xresnet1d18_deep": "112_models.XResNet1d.ipynb",
         "xresnet1d34_deep": "112_models.XResNet1d.ipynb",
         "xresnet1d50_deep": "112_models.XResNet1d.ipynb",
         "xresnet1d18_deeper": "112_models.XResNet1d.ipynb",
         "xresnet1d34_deeper": "112_models.XResNet1d.ipynb",
         "xresnet1d50_deeper": "112_models.XResNet1d.ipynb",
         "XResNet1dPlus": "112b_models.XResNet1dPlus.ipynb",
         "xresnet1d18plus": "112b_models.XResNet1dPlus.ipynb",
         "xresnet1d34plus": "112b_models.XResNet1dPlus.ipynb",
         "xresnet1d50plus": "112b_models.XResNet1dPlus.ipynb",
         "xresnet1d101plus": "112b_models.XResNet1dPlus.ipynb",
         "xresnet1d152plus": "112b_models.XResNet1dPlus.ipynb",
         "xresnet1d18_deepplus": "112b_models.XResNet1dPlus.ipynb",
         "xresnet1d34_deepplus": "112b_models.XResNet1dPlus.ipynb",
         "xresnet1d50_deepplus": "112b_models.XResNet1dPlus.ipynb",
         "xresnet1d18_deeperplus": "112b_models.XResNet1dPlus.ipynb",
         "xresnet1d34_deeperplus": "112b_models.XResNet1dPlus.ipynb",
         "xresnet1d50_deeperplus": "112b_models.XResNet1dPlus.ipynb",
         "TemporalBlock": "113_models.TCN.ipynb",
         "temporalconvnet": "113_models.TCN.ipynb",
         "TCN": "113_models.TCN.ipynb",
         "XCM": "114_models.XCM.ipynb",
         "TabModel": "120_models.TabModel.ipynb",
         "MultiInputNet": "130_models.MultiInputNet.ipynb"}

modules = ["utils.py",
           "data/validation.py",
           "data/preparation.py",
           "data/external.py",
           "data/core.py",
           "data/preprocessing.py",
           "data/transforms.py",
           "data/mixed_sample_augmentation.py",
           "data/image.py",
           "data/tabular.py",
           "data/mixed.py",
           "metrics.py",
           "learner.py",
           "optimizer.py",
           "callback/core.py",
           "callback/semi_supervised.py",
           "models/utils.py",
           "models/layers.py",
           "models/explainability.py",
           "models/ResNet.py",
           "models/ResNetPlus.py",
           "models/InceptionTime.py",
           "models/InceptionTimePlus.py",
           "models/MLP.py",
           "models/FCN.py",
           "models/FCNPlus.py",
           "models/ResCNN.py",
           "models/RNN.py",
           "models/RNNPlus.py",
           "models/XceptionTime.py",
           "models/XceptionTimePlus.py",
           "models/RNN_FCN.py",
           "models/RNN_FCNPlus.py",
           "models/TransformerModel.py",
           "models/TST.py",
           "models/TSTPlus.py",
           "models/OmniScaleCNN.py",
           "models/mWDN.py",
           "models/ROCKET.py",
           "models/XResNet1d.py",
           "models/XResNet1dPlus.py",
           "models/TCN.py",
           "models/XCM.py",
           "models/TabModel.py",
           "models/MultiInputNet.py"]

doc_url = "https://timeseriesAI.github.io/tsai/"

git_url = "https://github.com/timeseriesAI/tsai/tree/master/"

def custom_doc_links(name): return None
